program wget_workers;

uses workers_t_asynch, SysUtils, math, Classes, process, fgl;

function WGetDownload(Const URL : String; Amount: Integer) : TStringStream;
var
  Buffer : Array[0..4096] of byte;
  Count : Integer;  
begin
   result := TStringStream.Create(URL + ': ');
   with TProcess.Create(nil) do
    try
      Executable:='wget';
      with Parameters do
       begin
          Add('-q');
          Add('--output-document=-');
          Add(URL);
       end;
      Options:=[poUsePipes,poNoConsole];
      Execute;
      While Running and (Amount > 0) do
        begin
           Count:=Output.Read(Buffer,min(Amount,SizeOf(Buffer)));
           Amount := Amount - Count;
           If (Count>0) then begin
              WriteLn ('Read: ', Count);
              result.WriteBuffer(Buffer,Count);
           end
        end;
      If (ExitStatus<>0) then
        Exception.Create(Format('WGET %s: exit status %d',[URL,ExitStatus]));
    finally
      Free;
    end;
end;

type
   TJob = class
      url: String;
      output: String;
      constructor Create (aURL: String);
      procedure Execute;      
   end;
   constructor TJob.Create (aURL: String);
   begin url := aURL end;
   
   procedure TJob.Execute;
   var S: TStringStream;
   begin
      WriteLn ('Job: reading ', url);
      S :=  WGetDownload (url, 1000);
      S.Seek (0, soFromBeginning);
      output := S.ReadString (1000);
      S.Free;
   end;
type
   TJobList = specialize TFPGObjectList<TJob>;
var
   JobList: TJobList;
   Manager: TWorkManager;
   Job: TJob;
begin
   Manager := TWorkManager.Create (4);
   JobList := TJobList.Create;
   JobList.Add (TJob.Create ('http://www.textfiles.com/art/bones.art'));
   JobList.Add (TJob.Create ('http://www.textfiles.com/art/16ton'));
   JobList.Add (TJob.Create ('http://www.textfiles.com/100/dec.wars'));
   JobList.Add (TJob.Create ('http://www.textfiles.com/100/famous.bug'));
   JobList.Add (TJob.Create ('http://www.textfiles.com/100/hack11a.txt'));
   JobList.Add (TJob.Create ('http://pl.wikipedia.org/wiki/Aksjomaty_i_konstrukcje_liczb'));
   JobList.Add (TJob.Create ('http://www.freepascal.org/docs-html/rtl/classes/tstream.seek.html'));
   JobList.Add (TJob.Create ('http://www.freepascal.org/docs-html/fcl/process/tprocess.running.html'));
   for Job in JobList do Manager.AddJob (@Job.Execute);
   while Manager.Jobs > 0 do
      RtlEventWaitFor (Manager.ExecutedJob, 500);
   for Job in JobList do
   begin
      WriteLn ('Output: ', Job.url);
      WriteLn (Job.output);
   end;
   Manager.Free;
   JobList.Free;
end.   